-- SlashCommands --
local SlashCommands = FlexBar2.SlashCommands;
-- /fb show command
local function SlashCommand_Show(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	FlexBar2:ApplySetting(ButtonTable, "Show", true);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:Show();
		ButtonList[ButtonName]:Enable();
	end
end

-- /fb hide command
local function SlashCommand_Hide(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	FlexBar2:ApplySetting(ButtonTable, "Show", false);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:Hide();
		ButtonList[ButtonName]:Disable();
	end
end

-- /fb lock command
local function SlashCommand_Lock(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	FlexBar2:ApplySetting(ButtonTable, "Lock", true);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:Lock();
	end
end

-- /fb unlock command
local function SlashCommand_Unlock(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	FlexBar2:ApplySetting(ButtonTable, "Lock", false);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:Unlock();
	end
end

-- /fb setposition command
local function SlashCommand_SetPosition(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	local ButtonList = FlexBar2.Buttons;
	-- Only move if position was specified in "stupid" mode
	if(Params.smart == "false") then
		if(Params.x ~= nil) then FlexBar2:ApplySetting(ButtonTable, "X", Params.x); end
		if(Params.y ~= nil) then FlexBar2:ApplySetting(ButtonTable, "Y", Params.y); end
	elseif(Params.smart == "true") then
		-- In "smart" mode, use the coords of the first button and position the rest relative to it)
		if(Params.x == nil) then Params.x = ButtonList[ButtonTable[1]].InfoTable.X; end
		if(Params.y == nil) then Params.y = ButtonList[ButtonTable[1]].InfoTable.Y; end
		for Number, Button in ipairs(ButtonTable) do
			FlexBar2:ApplySetting(Button, "X", Params.x + (37 * (Number - ((math.ceil(Number / 12) - 1) * 12) - 1)));
			FlexBar2:ApplySetting(Button, "Y", Params.y - (37 * (math.ceil(Number / 12) - 1)));
		end
	end
	-- Loop through all buttons and call SetPosition without args, which causes it to anchor to its currently saved settings
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:SetPosition();
	end
end

--  /fb setscale command
local function SlashCommand_SetScale(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	local Scale = tonumber(Params.scale);
	FlexBar2:ApplySetting(ButtonTable, "Scale", Scale);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:SetScale(Scale);
	end
end

--  /fb setalpha command
local function SlashCommand_SetAlpha(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	local Alpha = tonumber(Params.alpha);
	FlexBar2:ApplySetting(ButtonTable, "Alpha", Alpha);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:SetScale(Alpha);
	end
end


-- /fb tocursor command
local function SlashCommand_ToCursor(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		local Button = ButtonList[ButtonName]
		X, Y = Button:FrameToCursor();
		FlexBar2:ApplySetting(ButtonName, "X", X);
		FlexBar2:ApplySetting(ButtonName, "Y", Y);
	end
end

-- /fb setbuttontype command
local function SlashCommand_SetButtonType(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:SetType(Params.type);
	end
end

local function SlashCommand_Line(Params)
	local Rows = tonumber(Params.rows);
	local Columns = tonumber(Params.columns);
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	local ButtonList = FlexBar2.Buttons;
	local StartX = ButtonList[ButtonTable[1]].InfoTable.X;
	local StartY = ButtonList[ButtonTable[1]].InfoTable.Y;
	local Space = tonumber(Params.space);
	-- Only rows specified
	if(Rows) then
		local LastScale = 1;
		local Row = 1;
		local LastX = StartX;
		local LastY = StartY;
		for Key, ButtonName in ipairs(ButtonTable) do
			-- Do nothing for the first button, we position the others relative to it
			if(Key ~= 1) then
				local Button = ButtonList[ButtonName];
				-- Go down a row if the current one is full
				if(Row == Rows) then
					Row = 1;
					LastY = LastY - Space;
					LastX = StartX;
				else
					LastX = LastX + Space;
					Row = Row + 1;
				end
				Button:SetPosition(LastX, LastY);
				Button.InfoTable.X = LastX;
				Button.InfoTable.Y = LastY;
			end
		end
	-- Only columns specified
	elseif(Columns) then
		local LastScale = 1;
		local Column = 1;
		local LastX = StartX;
		local LastY = StartY;
		local LastScale;
		for Key, ButtonName in ipairs(ButtonTable) do
			-- Do nothing for the first button, we position the others relative to it
			local Button = ButtonList[ButtonName];	
			if(Key ~= 1) then
				-- Go down a row if the current one is full
				if(Column == Columns) then
					Column = 1;
					LastX = LastX + Space * LastScale;
					LastY = StartY;
				else
					LastY = LastY - Space * LastScale;
					Column = Column + 1;
				end
				Button:SetPosition(LastX, LastY);
				Button.InfoTable.X = LastX;
				Button.InfoTable.Y = LastY;
			end
			LastScale = Button:GetScale();
		end
	else
		FlexBar2:Print("Need either number of rows or number of columns declared");
	end
end

local function SlashCommand_SetTexture(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		FlexBar2:Print(Params.texture);
		local Button = ButtonList[ButtonName];
		Button.InfoTable.Texture = Params.texture;
		Button:SetIcon(Params.texture);
	end
end

-- Localize this neat verify function for faster access
local VerifyExistance = FlexBar2.SlashCommands.VerifyExistance;
-- AddCommand format :AddCommand(Command, Function, Group, Help)
-- AddParam format :AddParam(Command, Param, Required, AllowedValues, DefaultValue, VerifyFunc) 
-- everything here should be self explainative knowing the syntax
SlashCommands:AddCommand("show", SlashCommand_Show, "layout", "button=<Button>");
	SlashCommands:AddParam("show", "button", true, nil, nil, VerifyExistance)
SlashCommands:AddCommand("hide", SlashCommand_Hide, "layout", "button=<Button>");
	SlashCommands:AddParam("hide", "button", true, nil, nil, VerifyExistance)
SlashCommands:AddCommand("lock", SlashCommand_Lock, "layout", "button=<Button>");
	SlashCommands:AddParam("lock", "button", true, nil, nil, VerifyExistance)
SlashCommands:AddCommand("unlock", SlashCommand_Unlock, "layout", "button=<Button>");
	SlashCommands:AddParam("unlock", "button", true, nil, nil, VerifyExistance)
SlashCommands:AddCommand("setposition", SlashCommand_SetPosition, "layout", "button=<Button> x=<x coordinate> y=<y coordinate> (smart=<true/false>)");
SlashCommands:SetAlias("setposition", "position");
	SlashCommands:AddParam("setposition", "button", true, nil, nil, VerifyExistance)
	SlashCommands:AddParam("setposition", "x", false, nil, nil, function(X) if(string.find(X, "^%d*%.?%d+$")) then return true; else return "Invalid X"; end end);
	SlashCommands:AddParam("setposition", "y", false, nil, nil, function(Y) if(string.find(Y, "^%d*%.?%d+$")) then return true; else return "Invalid Y"; end end);
	SlashCommands:AddParam("setposition", "smart", false, {"true", "false"}, "true", nil);
SlashCommands:AddCommand("setscale", SlashCommand_SetScale, "layout", "button=<Button> scale=<Scale>");
SlashCommands:SetAlias("setscale", "scale");
    SlashCommands:AddParam("setscale", "button", true, nil, nil, VerifyExistance)
    SlashCommands:AddParam("setscale", "scale", false, nil, "1", function(Scale) if(string.find(Scale, "^%d*%.?%d+$")) then return true; else return "Invalid scale"; end end);
SlashCommands:AddCommand("setalpha", SlashCommand_SetAlpha, "layout", "button=<Button> scale=<Scale>");
SlashCommands:SetAlias("setalpha", "alpha");
    SlashCommands:AddParam("setalpha", "button", true, nil, nil, VerifyExistance)
    SlashCommands:AddParam("setalpha", "alpha", false, nil, "1", function(Alpha) if(string.find(Alpha, "^%d*%.?%d+$")) then return true; else return "Invalid alpha"; end end);
SlashCommands:AddCommand("tocursor", SlashCommand_ToCursor, "layout", "button=<Button>");
SlashCommands:AddCommand("setbuttontype", SlashCommand_SetButtonType, "layout", "button=<Button> type=<Type>");
SlashCommands:SetAlias("setbuttontype", "buttontype");
	SlashCommands:AddParam("setbuttontype", "button", true, nil, nil, VerifyExistance);
	SlashCommands:AddParam("setbuttontype", "type", true, nil, nil, nil);
SlashCommands:AddCommand("line", SlashCommand_Line, "layout", "button=<Button> (rows=X) (Columns=Y) (space=Z)");
	SlashCommands:AddParam("line", "button",  true, nil, nil, VerifyExistance);
	SlashCommands:AddParam("line", "rows", false, nil, nil, function(Rows) return tonumber(Rows) ~= nil end);
	SlashCommands:AddParam("line", "columns", false, nil, nil, function(Columns) return tonumber(Columns) ~= nil end);
	SlashCommands:AddParam("line", "space", false, nil, "37", function(Space) return tonumber(Space) ~= nil end);
SlashCommands:AddCommand("settexture", SlashCommand_SetTexture, "layout", "texture=<texture>");
SlashCommands:SetAlias("settexture", "texture");
	SlashCommands:AddParam("settexture", "button",  true, nil, nil, VerifyExistance);
	SlashCommands:AddParam("settexture", "texture", false, nil, nil, nil);
