-- SlashCommands --
local SlashCommands = FlexBar2.SlashCommands;

-- /fb setbinding command
local function SlashCommand_SetBinding(Params)
    local ButtonTable = FlexBar2:ToButtonTable(Params.button);
    -- ToButtonTable command abuse, it serves us well
    local BindingsList = FlexBar2:ToButtonTable(Params.binding);
    local ButtonList = FlexBar2.Buttons;
    if(#ButtonTable ~= #BindingsList) then FlexBar2:Print("number of bindings given does not match the numbers of buttons given"); return; end
    for Key, ButtonName in ipairs(ButtonTable) do
       ButtonList[ButtonName]:SetBinding(BindingsList[Key]);
    end
end

-- /fb clearbinding command
local function SlashCommand_ClearBinding(Params)
    local ButtonTable = FlexBar2:ToButtonTable(Params.button);
    local ButtonList = FlexBar2.Buttons;
    for _, ButtonName in ipairs(ButtonTable) do
        ButtonList[ButtonName]:SetBinding();
    end
end

-- Localize this neat verify function for faster access
local VerifyExistance = FlexBar2.SlashCommands.VerifyExistance;

-- Helper verification function
local VerifyBinding = function(BindingString)
	local BindingList = FlexBar2:ToButtonTable(BindingString);
	for _, Binding in ipairs(BindingList) do
		if(Binding == "MM") then Binding = "BUTTON3"; end
		if(not FlexBar2:GetModule("Keybindings"):ValidateBinding(Binding)) then
			return "Invalid keybinding";
		end
	end
	return true;
end

-- AddCommand format :AddCommand(Command, Function, Group, Help)
-- AddParam format :AddParam(Command, Param, Required, AllowedValues, DefaultValue, VerifyFunc) 
-- everything here should be self explainative knowing the syntax
SlashCommands:AddCommand("setbinding", SlashCommand_SetBinding, "keybinding", "button=<Button> binding=<Key>");
SlashCommands:SetAlias("setbinding", "bind");
SlashCommands:SetAlias("setbinding", "binding");
	SlashCommands:AddParam("setbinding", "button", true, nil, nil, VerifyExistance);
	SlashCommands:AddParam("setbinding", "binding", true, nil, nil, VerifyBinding);

SlashCommands:AddCommand("clearbinding", SlashCommand_ClearBinding, "keybinding", "button=<Button>");
SlashCommands:SetAlias("clearbinding", "unbind");
	SlashCommands:AddParam("clearbinding", "button", true, nil, nil, function(Button) return FlexBar2:ButtonsExist(Button); end);


